import {
  Any,
  ContentSourceMap,
  ContentSourceMapDocument,
  ContentSourceMapDocumentBase,
  ContentSourceMapDocuments,
  ContentSourceMapDocumentValueSource,
  ContentSourceMapLiteralSource,
  ContentSourceMapMapping,
  ContentSourceMapMappings,
  ContentSourceMapParsedPath,
  ContentSourceMapParsedPathKeyedSegment,
  ContentSourceMapPaths,
  ContentSourceMapRemoteDocument,
  ContentSourceMapSource,
  ContentSourceMapUnknownSource,
  ContentSourceMapValueMapping,
  IndexTuple,
  KeyedSegment,
  Path,
  PathSegment,
  SanityDocument,
} from '@sanity/client/csm'
import type {ForwardRefExoticComponent, HTMLProps, SVGProps} from 'react'

export {Any}

export {ContentSourceMap}

export {ContentSourceMapDocument}

export {ContentSourceMapDocumentBase}

export {ContentSourceMapDocuments}

export {ContentSourceMapDocumentValueSource}

export {ContentSourceMapLiteralSource}

export {ContentSourceMapMapping}

export {ContentSourceMapMappings}

export {ContentSourceMapParsedPath}

export {ContentSourceMapParsedPathKeyedSegment}

export {ContentSourceMapPaths}

export {ContentSourceMapRemoteDocument}

export {ContentSourceMapSource}

export {ContentSourceMapUnknownSource}

export {ContentSourceMapValueMapping}

/**
 * Transforms sanity node data into an encoded string
 * @param node - An unencoded sanity node object
 * @returns An encoded string of sanity data
 * @public
 */
export declare function encodeSanityNodeData(node: SanityNode): string | undefined

export declare const htmlElements: readonly [
  'a',
  'abbr',
  'address',
  'area',
  'article',
  'aside',
  'audio',
  'b',
  'base',
  'bdi',
  'bdo',
  'big',
  'blockquote',
  'body',
  'br',
  'button',
  'canvas',
  'caption',
  'cite',
  'code',
  'col',
  'colgroup',
  'data',
  'datalist',
  'dd',
  'del',
  'details',
  'dfn',
  'dialog',
  'div',
  'dl',
  'dt',
  'em',
  'embed',
  'fieldset',
  'figcaption',
  'figure',
  'footer',
  'form',
  'h1',
  'h2',
  'h3',
  'h4',
  'h5',
  'h6',
  'head',
  'header',
  'hgroup',
  'hr',
  'html',
  'i',
  'iframe',
  'img',
  'input',
  'ins',
  'kbd',
  'keygen',
  'label',
  'legend',
  'li',
  'link',
  'main',
  'map',
  'mark',
  'menu',
  'menuitem',
  'meta',
  'meter',
  'nav',
  'noscript',
  'object',
  'ol',
  'optgroup',
  'option',
  'output',
  'p',
  'param',
  'picture',
  'pre',
  'progress',
  'q',
  'rp',
  'rt',
  'ruby',
  's',
  'samp',
  'script',
  'section',
  'select',
  'small',
  'source',
  'span',
  'strong',
  'style',
  'sub',
  'summary',
  'sup',
  'table',
  'tbody',
  'td',
  'textarea',
  'tfoot',
  'th',
  'thead',
  'time',
  'title',
  'tr',
  'track',
  'u',
  'ul',
  'var',
  'video',
  'wbr',
  'webview',
]

export {IndexTuple}

export {KeyedSegment}

export {Path}

export {PathSegment}

export declare const sanity: SanityElements

export {SanityDocument}

export declare interface SanityElementProps {
  children?: SourceNode | null
}

export declare type SanityElements = SanityHTMLElements & SanitySVGElements

export declare type SanityHTMLElements = Record<
  (typeof htmlElements)[number],
  ForwardRefExoticComponent<SanityElementProps & Omit<HTMLProps<HTMLElement>, 'children' | 'ref'>>
>

/** @public */
export declare type SanityKey =
  | '_createdAt'
  | '_dataset'
  | '_id'
  | '_key'
  | '_originalId'
  | '_projectId'
  | '_ref'
  | '_rev'
  | '_strengthenOnPublish'
  | '_type'
  | '_updatedAt'
  | '_weak'

/**
 * Data resolved from a Sanity node
 * @public
 */
export declare type SanityNode = {
  baseUrl: string
  dataset?: string
  id: string
  isDraft?: string
  path: string
  projectId?: string
  tool?: string
  type?: string
  workspace?: string
}

/** @public */
export declare interface SanityNodeContext {
  baseUrl: string
  tool?: string
  workspace?: string
}

/** @public */
export declare type SanityPrimitive = string | number | boolean | null

export declare type SanitySVGElements = Record<
  (typeof svgElements)[number],
  ForwardRefExoticComponent<SanityElementProps & Omit<SVGProps<SVGElement>, 'children' | 'ref'>>
>

/** @public */
export declare type SourceNode<
  T extends Exclude<SanityPrimitive, null> = Exclude<SanityPrimitive, null>,
> = {
  $$type$$: 'sanity'
  path: string | undefined
  source: SanityNode | undefined
  value: T
}

export declare const svgElements: readonly [
  'animate',
  'circle',
  'defs',
  'desc',
  'ellipse',
  'g',
  'image',
  'line',
  'filter',
  'marker',
  'mask',
  'metadata',
  'path',
  'pattern',
  'polygon',
  'polyline',
  'rect',
  'stop',
  'svg',
  'switch',
  'symbol',
  'text',
  'tspan',
  'use',
  'view',
  'clipPath',
  'feBlend',
  'feColorMatrix',
  'feComponentTransfer',
  'feComposite',
  'feConvolveMatrix',
  'feDiffuseLighting',
  'feDisplacementMap',
  'feDistantLight',
  'feDropShadow',
  'feFlood',
  'feFuncA',
  'feFuncB',
  'feFuncG',
  'feFuncR',
  'feGaussianBlur',
  'feImage',
  'feMerge',
  'feMergeNode',
  'feMorphology',
  'feOffset',
  'fePointLight',
  'feSpecularLighting',
  'feSpotLight',
  'feTile',
  'feTurbulence',
  'foreignObject',
  'linearGradient',
  'radialGradient',
  'textPath',
]

/** @public */
export declare function unwrapData<T = any, S extends WrappedValue<T> = WrappedValue<T>>(
  wrapper: S,
): UnwrappedValue<S>

/** @public */
export declare type UnwrappedValue<W = WrappedValue<unknown>> =
  W extends SourceNode<string>
    ? string
    : W extends SourceNode<number>
      ? number
      : W extends SourceNode<boolean>
        ? boolean
        : W extends Array<unknown>
          ? Array<UnwrappedValue<W[number]>>
          : W extends {}
            ? {
                [P in keyof W]: P extends SanityKey ? W[P] : UnwrappedValue<W[P]>
              }
            : W extends string
              ? string
              : W extends null
                ? null
                : W extends undefined
                  ? undefined
                  : never

/** @public */
export declare function wrapData<T>(
  context: SanityNodeContext,
  value: T,
  sourceMap: ContentSourceMap | undefined,
  resultPath?: ContentSourceMapParsedPath,
  keyedResultPath?: ContentSourceMapParsedPath,
): WrappedValue<T>

/** @public */
export declare type WrappedValue<T> = T extends string
  ? SourceNode<string>
  : T extends number
    ? SourceNode<number>
    : T extends boolean
      ? SourceNode<boolean>
      : T extends Array<unknown>
        ? Array<WrappedValue<T[number]>>
        : T extends {}
          ? {
              [P in keyof T]: P extends SanityKey ? T[P] : WrappedValue<T[P]>
            }
          : T extends string
            ? string
            : T extends null
              ? null
              : T extends undefined
                ? undefined
                : never

export {}
